/*!
 * @file ovinotificationinterface.h
 * @brief Ovi Notification session class represents the session between the application and Notifications Enabler.
 *
 * This file is part of Notifications API
 * <p>
 * @copyright  Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 *                   Contact:  notifications.support@nokia.com
 *                  This software, including documentation, is protected by copyright controlled by Nokia Corporation.
 *                  All rights reserved. You may use this file only in accordance with the Nokia Developer Software
 *                  Agreement version 2.0 and accompanying Ovi any additional terms. This material also contains
 *                  confidential information which may not be disclosed to others without the prior written consent of Nokia.
 *
 * @license Nokia Developer Software Agreement version 2.0 and accompanying Ovi any additional terms
 * </p>
 */


#ifndef OVINOTIFICATIONS_INTERFACE_H
#define OVINOTIFICATIONS_INTERFACE_H

// System includes
#include <QtCore>
#include <QString>
#include <qglobal.h>

#if defined Q_OS_SYMBIAN
#define ONEPLUGIN      "c:/resource/qt/plugins/oneplugin.qtplugin"
#elif defined(Q_OS_WIN32) || defined(Q_OS_WIN64)
#define ONEPLUGIN      "oneplugin.dll"
#elif defined Q_OS_MAC
#define ONEPLUGIN      "liboneplugin.dylib"
#elif defined Q_OS_LINUX  && defined QT_SIMULATOR
#define ONEPLUGIN      "liboneplugin.so"
#else // meego
#define ONEPLUGIN      "/usr/lib/qt4/plugins/liboneplugin.so"
#endif

// Forward Declarations
class MbModuleInterface;
static const char* KOviNotificationInterfaceId = "com.nokia.OviNotificationSession/1.0";
// Constants

/*!
 *
 * Main class of the Ovi Notification service provider.
 *
 *
 * Code example of how Qt applications uses this API.
 *
 @verbatim
 // OviNotificationSession *iNotificationSession;

 // Load service object
 QPluginLoader* loader = new QPluginLoader(ONEPLUGIN);
 iNotificationSession = static_cast<OviNotificationSession*>(loader->instance());

 // Connect signals from session

 // State enumerations is emitted from stateChanged(QObject*) signal.
 connect(iNotificationSession,SIGNAL(stateChanged(QObject*)), SLOT(sessionStateChanged(QObject*)));

 // Incoming notification objects is emitted from received(QObject*) signal.
 connect(iNotificationSession,SIGNAL(received(QObject)), SLOT(receivedClb(QObject)));

 // Register application. This is an asynchronous call.
 // OviNotificationSession::stateChanged signal is emitted with state EStateOnline
 // when application is successfully registered.
 iNotificationSession->registerApplication(aApplicationId, aWakeup);
 }

 // When message is received , received(QObject) signal is emitted.
 OviNotificationMessage* message = static_cast<OviNotificationMessage*>(obj);

 message->from();
 message->senderInformation();
 message->timestamp();

 }

 @endverbatim
 *
 * Client application should connect all signals from Ovi Notification service object to
 * receive callbacks. See signals in OviNotificationSession.
 *
 */

class OviNotificationInterface
{
public:



    /*! Registers Notifications Enabler application. This is an asynchronous method.
     *  It is mandatory to call this method before Ovi Notifications can be used.
     *  If registration is succesfull Notifications Enabler will open connection Notification Service
     *  if not open already.
     *  OviNotificationSession::stateChanged signal is emitted with EStateConnecting and EStateOnline after
     *  succesfull registration. Otherwise one of the OviNotificationState::StateError
     *  errors is emited.
     *
     * @param aApplicationId Application's Identifier
     */
    virtual void registerApplication(const QString& aApplicationId) = 0;

    /*! Setter for the wakeup of the application. This is an asynchronous method.
     * This function should be called when session state is OviNotificationState::EStateOnline.
     * The stateChanged(QObject* ) signal would be emitted with OviNotificationState::StateError
     * OviNotificationState::EErrorNotAllowed in the following cases
     *  -# If this function is called in state other than OviNotificationState::EStateOnline.
     *  -# If the function is trying to set value before the previous value has not been saved in the Notifications Enabler.
     *
     * If this flag is set and the applicatin receives notifications
     * while application is not running then
     *  -#if notificationUi is true - user will receive notification  popup
     *  informing that this application has received notification,user needs
     *  to select the popup and it will lauch this application.
     *  -#if notification Ui is false - Application will directly launch. To
     *  receive the notification, application must call registerApplication()
     *  after launching.
     *  -#In case the application is running and notification arrives for the
     *  application it is directly delivered to the application.
     *  Stores the wakeup related settings in the stack.
     *
     * @param aWakeUp  If true will enable the wakeup feature else will disable
     *                 wakeup feature.
     */
    virtual void setWakeUp(bool aWakeUp) = 0;

    /*! Setter for the Notification Ui
     * This function should be called when session state is OviNotificationState::EStateOnline.
     * The stateChanged(QObject* ) signal would be emitted with OviNotificationState::StateError
     * OviNotificationState::EErrorNotAllowed in the following cases
     *  -# If this function is called in state other than OviNotificationState::EStateOnline.
     *  -# If the function is trying to set value before the previous value has not been saved in the Notifications Enabler.
     *
     * @param aUseNotificationUi    If set to true then the Notification UI is shown in the events of new messages when the
     *                              application is not running.
     * @param aIcon                 The icon data to be shown for this application in the notification ui.
     *                              The icon format should be Base64 encoded image data read from the file,
     *                              supported formats svg, png, img, bmp,jpg
     *  Returns true if the NotificationUi setting has been succesful else returns false in cases of
     *  -# Too large Icon
     *
     *  \b Note: setWakeUp must be set that setNotificationUi would affect 
     */
    virtual bool setNotificationUi(bool aUseNotificationUi, const QString& aIcon = "") = 0;

    /*! Getter for the wakeup.
     * This function should be called when session state is OviNotificationState::EStateOnline.
     * If the function is called in any other state then stateChanged signal would be emitted with
     * OviNotificationState::EErrorNotAllowed which indicates the values returned by this
     * function are not valid.
     *
     *  Returns true if the wakeup setting is true otherwise returns false.
     */
    virtual bool isWakeUp() = 0;

    /*! Getter for the notification UI.
     * This function should be called when session state is OviNotificationState::EStateOnline.
     * If the function is called in any other state then stateChanged signal would be emitted with
     * OviNotificationState::EErrorNotAllowed which indicates the values returned by this
     * function are not valid.
     *  Returns true if the NotificationUi setting is true otherwise returns false.
     */
    virtual bool isNotificationUi() = 0;

    /*! Asynchronous call used to unregister application.
     *  Should be called when application wants to close and no longer wishes to receive the
     *  notifications from the from Ovi Notification service.
     *  Removes all application related information and stored settings. Like
     *  -# Wakeup
     *  -# NotificationUi
     *
     *  If application is already unregistered or is not registered at all,
     *  OviNotificationState::EErrorNotAllowed is emited in
     *  OviNotificationSession::stateChanged() signal.
     */
    virtual void unregisterApplication() = 0;

    /*! Retrieve user information used by Notifications Enabler. Requested information is returned by
     *  emitting OviNotificationSession::notificationInformation(QObject* ) signal.
     *  The information contains the user account used by application. This value could be empty
	 *  in case of the service is not available or the request has timedout.
	 *  The signal stateChanged(QObject* ) with OviNotificationState::StateError
	 *  OviNotificationState::EErrorInvalidServiceId is emitted if the passed ServiceId is invalid.
     *
     * This function should be called when session state is OviNotificationState::EStateOnline.
     * The stateChanged(QObject* ) signal would be emitted with OviNotificationState::StateError
     * OviNotificationState::EErrorNotAllowed in the following cases
     *  -# If this function is called in state other than OviNotificationState::EStateOnline.
     *  -# If the function is trying to call before the previous request has not been processed by the Notifications Enabler.
     *
     * @param aServiceId    The identifier of the Notification Service
     */
    virtual void getNotificationInformation(const QString& aServiceId = "") = 0;

    /*! Retrieve version information.
     *  Requested version information is returned in
     *  OviNotificationSession::version() signal.
     */
     virtual void getVersion() = 0;
};

Q_DECLARE_INTERFACE(OviNotificationInterface, KOviNotificationInterfaceId);

#endif // OVINOTIFICATIONS_INTERFACE_H

// End of file
