/*!
 * @file ovinotificationstate.h
 * @brief OviNotificationState is the state of the Session
 *
 * This file is part of Notifications API
 * <p>
 * @copyright  Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 *                   Contact:  notifications.support@nokia.com
 *                  This software, including documentation, is protected by copyright controlled by Nokia Corporation.
 *                  All rights reserved. You may use this file only in accordance with the Nokia Developer Software
 *                  Agreement version 2.0 and accompanying Ovi any additional terms. This material also contains
 *                  confidential information which may not be disclosed to others without the prior written consent of Nokia.
 *
 * @license Nokia Developer Software Agreement version 2.0 and accompanying Ovi any additional terms
 * </p>
 */


#ifndef OVINOTIFICATIONSTATE_H
#define OVINOTIFICATIONSTATE_H

#include <QObject>
#include <ovinotificationinterface.h>

// forward declaration
class OviNotificationStatePrivate;
class OviNotificationResponseHandler;
class OviNotificationSessionPrivate;

class OviNotificationState : public QObject
{
    Q_OBJECT
    Q_ENUMS(State)
    Q_PROPERTY(State sessionState READ sessionState)
    Q_ENUMS(StateError)
    Q_PROPERTY(StateError sessionError READ sessionError)


public:
    OviNotificationState();
    ~OviNotificationState();

    /*!
     * Session states.
     */
    enum State
    {
        EStateOffline                   = 0,    //!< No connection to XMPP core
        EStateConnecting                = 1,    //!< Notifications Enabler is connecting, for the first time or after temporarily loosing connection
        EStateOnline                    = 2    //!< Application has connected, it can receive messages
    };

    enum StateError
    {
        EErrorNone                  = 0,  //!< Used when succesfully changing session state
        EErrorEnablerNotFound       = 1,  //!< Notifications Enabler cannot be found
        EErrorApplicationIdConflict = 2,  //!< Conflicting application id, same application id has already been registered
        EErrorMissingCredentials    = 3,  //!< Notifications Enabler cannot get credentials to connect to server
        EErrorNoNetwork             = 4,  //!< Network connection lost
        EErrorServiceUnavailable    = 5,  //!< Ovi Notification Service is no longer available because service's authentication has expired
        EErrorNotAllowed            = 6,  //!< A method that is not allowed to be called in the state was called
        EErrorDisabledByUser        = 7,  //!<User has disabled all Ovi Notifications
        EErrorNotificationIdInvalidated = 8, //!< User account has been changed
        EErrorWidgetIdMismatch      = 9,   //!< Widget id is missing from widget XML configuration file file (info.plist/config.xml)
        EErrorInvalidApplicationId  = 10,  //!< Application id is invalid, too long applicationId or empty application id
        EErrorInvalidServiceId      = 11,   //!< Returned when trying to request Notification Id with servie ID that ONE server does not accept
        EErrorEnablerExpired        = 12,   //!< Returned when Notifications Enabler has expired
    };

public slots:
    /*!
     * Returns the state.
     *
     * @return OviNotificationInterface::State The state of the session
     */
    virtual State sessionState();

    /*!
     * Returns the state as string.
     *
     * @return QString OviNotificationInterface::State as string
     */
    virtual QString sessionStateString();

    /*!
     * Returns the error
     *
     * @return OviNotificationInterface::StateError The error of the session
     */
    virtual StateError sessionError();

    /*!
     * Returns the error as integer
     *
     * @return int The error of the session
     */
    virtual int sessionErrorInteger();

    /*!
     * Returns the error as string
     *
     * @return QString OviNotificationInterface::StateError as string
     */
    virtual QString sessionErrorString();

private:
     OviNotificationStatePrivate* impl;

     // Friend class
     friend class OviNotificationResponseHandler;
     friend class OviNotificationSessionPrivate;

};

#endif // OVINOTIFICATIONSTATE_H
